

from __future__ import division
from __future__ import absolute_import
#typing

import torch

from allennlp.common.checks import check_dimensions_match
from allennlp.data import Vocabulary
from allennlp.models.model import Model
from allennlp.modules import FeedForward, InputVariationalDropout
from allennlp.modules.matrix_attention.legacy_matrix_attention import LegacyMatrixAttention
from allennlp.modules import Seq2SeqEncoder, SimilarityFunction, TextFieldEmbedder
from allennlp.nn import InitializerApplicator, RegularizerApplicator
from allennlp.nn.util import get_text_field_mask, last_dim_softmax, weighted_sum, replace_masked_values
from allennlp.training.metrics import CategoricalAccuracy


class ESIM(Model):
    u"""
    This ``Model`` implements the ESIM sequence model described in `"Enhanced LSTM for Natural Language Inference"
    <https://www.semanticscholar.org/paper/Enhanced-LSTM-for-Natural-Language-Inference-Chen-Zhu/83e7654d545fbbaaf2328df365a781fb67b841b4>`_
    by Chen et al., 2017.

    Parameters
    ----------
    vocab : ``Vocabulary``
    text_field_embedder : ``TextFieldEmbedder``
        Used to embed the ``premise`` and ``hypothesis`` ``TextFields`` we get as input to the
        model.
    encoder : ``Seq2SeqEncoder``
        Used to encode the premise and hypothesis.
    similarity_function : ``SimilarityFunction``
        This is the similarity function used when computing the similarity matrix between encoded
        words in the premise and words in the hypothesis.
    projection_feedforward : ``FeedForward``
        The feedforward network used to project down the encoded and enhanced premise and hypothesis.
    inference_encoder : ``Seq2SeqEncoder``
        Used to encode the projected premise and hypothesis for prediction.
    output_feedforward : ``FeedForward``
        Used to prepare the concatenated premise and hypothesis for prediction.
    output_logit : ``FeedForward``
        This feedforward network computes the output logits.
    dropout : ``float``, optional (default=0.5)
        Dropout percentage to use.
    initializer : ``InitializerApplicator``, optional (default=``InitializerApplicator()``)
        Used to initialize the model parameters.
    regularizer : ``RegularizerApplicator``, optional (default=``None``)
        If provided, will be used to calculate the regularization penalty during training.
    """
    def __init__(self, vocab            ,
                 text_field_embedder                   ,
                 encoder                ,
                 similarity_function                    ,
                 projection_feedforward             ,
                 inference_encoder                ,
                 output_feedforward             ,
                 output_logit             ,
                 dropout        = 0.5,
                 initializer                        = InitializerApplicator(),
                 regularizer                                  = None)        :
        super(ESIM, self).__init__(vocab, regularizer)

        self._text_field_embedder = text_field_embedder
        self._encoder = encoder

        self._matrix_attention = LegacyMatrixAttention(similarity_function)
        self._projection_feedforward = projection_feedforward

        self._inference_encoder = inference_encoder

        if dropout:
            self.dropout = torch.nn.Dropout(dropout)
            self.rnn_input_dropout = InputVariationalDropout(dropout)
        else:
            self.dropout = None
            self.rnn_input_dropout = None

        self._output_feedforward = output_feedforward
        self._output_logit = output_logit

        self._num_labels = vocab.get_vocab_size(namespace=u"labels")

        check_dimensions_match(text_field_embedder.get_output_dim(), encoder.get_input_dim(),
                               u"text field embedding dim", u"encoder input dim")
        check_dimensions_match(encoder.get_output_dim() * 4, projection_feedforward.get_input_dim(),
                               u"encoder output dim", u"projection feedforward input")
        check_dimensions_match(projection_feedforward.get_output_dim(), inference_encoder.get_input_dim(),
                               u"proj feedforward output dim", u"inference lstm input dim")

        self._accuracy = CategoricalAccuracy()
        self._loss = torch.nn.CrossEntropyLoss()

        initializer(self)

    def forward(self,  # type: ignore
                premise                             ,
                hypothesis                             ,
                label                  = None,
                metadata                       = None  # pylint:disable=unused-argument
               )                           :
        # pylint: disable=arguments-differ
        u"""
        Parameters
        ----------
        premise : Dict[str, torch.LongTensor]
            From a ``TextField``
        hypothesis : Dict[str, torch.LongTensor]
            From a ``TextField``
        label : torch.IntTensor, optional (default = None)
            From a ``LabelField``
        metadata : ``List[Dict[str, Any]]``, optional, (default = None)
            Metadata containing the original tokenization of the premise and
            hypothesis with 'premise_tokens' and 'hypothesis_tokens' keys respectively.

        Returns
        -------
        An output dictionary consisting of:

        label_logits : torch.FloatTensor
            A tensor of shape ``(batch_size, num_labels)`` representing unnormalised log
            probabilities of the entailment label.
        label_probs : torch.FloatTensor
            A tensor of shape ``(batch_size, num_labels)`` representing probabilities of the
            entailment label.
        loss : torch.FloatTensor, optional
            A scalar loss to be optimised.
        """
        embedded_premise = self._text_field_embedder(premise)
        embedded_hypothesis = self._text_field_embedder(hypothesis)
        premise_mask = get_text_field_mask(premise).float()
        hypothesis_mask = get_text_field_mask(hypothesis).float()

        # apply dropout for LSTM
        if self.rnn_input_dropout:
            embedded_premise = self.rnn_input_dropout(embedded_premise)
            embedded_hypothesis = self.rnn_input_dropout(embedded_hypothesis)

        # encode premise and hypothesis
        encoded_premise = self._encoder(embedded_premise, premise_mask)
        encoded_hypothesis = self._encoder(embedded_hypothesis, hypothesis_mask)

        # Shape: (batch_size, premise_length, hypothesis_length)
        similarity_matrix = self._matrix_attention(encoded_premise, encoded_hypothesis)

        # Shape: (batch_size, premise_length, hypothesis_length)
        p2h_attention = last_dim_softmax(similarity_matrix, hypothesis_mask)
        # Shape: (batch_size, premise_length, embedding_dim)
        attended_hypothesis = weighted_sum(encoded_hypothesis, p2h_attention)

        # Shape: (batch_size, hypothesis_length, premise_length)
        h2p_attention = last_dim_softmax(similarity_matrix.transpose(1, 2).contiguous(), premise_mask)
        # Shape: (batch_size, hypothesis_length, embedding_dim)
        attended_premise = weighted_sum(encoded_premise, h2p_attention)

        # the "enhancement" layer
        premise_enhanced = torch.cat(
                [encoded_premise, attended_hypothesis,
                 encoded_premise - attended_hypothesis,
                 encoded_premise * attended_hypothesis],
                dim=-1
        )
        hypothesis_enhanced = torch.cat(
                [encoded_hypothesis, attended_premise,
                 encoded_hypothesis - attended_premise,
                 encoded_hypothesis * attended_premise],
                dim=-1
        )

        # The projection layer down to the model dimension.  Dropout is not applied before
        # projection.
        projected_enhanced_premise = self._projection_feedforward(premise_enhanced)
        projected_enhanced_hypothesis = self._projection_feedforward(hypothesis_enhanced)

        # Run the inference layer
        if self.rnn_input_dropout:
            projected_enhanced_premise = self.rnn_input_dropout(projected_enhanced_premise)
            projected_enhanced_hypothesis = self.rnn_input_dropout(projected_enhanced_hypothesis)
        v_ai = self._inference_encoder(projected_enhanced_premise, premise_mask)
        v_bi = self._inference_encoder(projected_enhanced_hypothesis, hypothesis_mask)

        # The pooling layer -- max and avg pooling.
        # (batch_size, model_dim)
        v_a_max, _ = replace_masked_values(
                v_ai, premise_mask.unsqueeze(-1), -1e7
        ).max(dim=1)
        v_b_max, _ = replace_masked_values(
                v_bi, hypothesis_mask.unsqueeze(-1), -1e7
        ).max(dim=1)

        v_a_avg = torch.sum(v_ai * premise_mask.unsqueeze(-1), dim=1) / torch.sum(
                premise_mask, 1, keepdim=True
        )
        v_b_avg = torch.sum(v_bi * hypothesis_mask.unsqueeze(-1), dim=1) / torch.sum(
                hypothesis_mask, 1, keepdim=True
        )

        # Now concat
        # (batch_size, model_dim * 2 * 4)
        v_all = torch.cat([v_a_avg, v_a_max, v_b_avg, v_b_max], dim=1)

        # the final MLP -- apply dropout to input, and MLP applies to output & hidden
        if self.dropout:
            v_all = self.dropout(v_all)

        output_hidden = self._output_feedforward(v_all)
        label_logits = self._output_logit(output_hidden)
        label_probs = torch.nn.functional.softmax(label_logits, dim=-1)

        output_dict = {u"label_logits": label_logits, u"label_probs": label_probs}

        if label is not None:
            loss = self._loss(label_logits, label.long().view(-1))
            self._accuracy(label_logits, label)
            output_dict[u"loss"] = loss

        return output_dict

    def get_metrics(self, reset       = False)                    :
        return {u'accuracy': self._accuracy.get_metric(reset)}

ESIM = Model.register(u"esim")(ESIM)



u"""
Utilities for working with the local dataset cache.
"""




from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
from builtins import str
import os
import logging
import shutil
import tempfile
import json
try:
    from urlparse import urlparse
except:
    from urllib.parse import urlparse
import tempfile
#typing
from hashlib import sha256
from functools import wraps

import boto3
from botocore.exceptions import ClientError
import requests

from allennlp.common.tqdm import Tqdm
from io import open

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name

CACHE_ROOT = tempfile.gettempdir()
DATASET_CACHE = str(CACHE_ROOT +'/'+ u"datasets")


def url_to_filename(url     , etag      = None)       :
    u"""
    Convert `url` into a hashed filename in a repeatable way.
    If `etag` is specified, append its hash to the url's, delimited
    by a period.
    """
    url_bytes = url.encode(u'utf-8')
    url_hash = sha256(url_bytes)
    filename = url_hash.hexdigest()

    if etag:
        etag_bytes = etag.encode(u'utf-8')
        etag_hash = sha256(etag_bytes)
        filename += u'.' + etag_hash.hexdigest()

    return filename


def filename_to_url(filename     , cache_dir      = None)                   :
    u"""
    Return the url and etag (which may be ``None``) stored for `filename`.
    Raise ``FileNotFoundError`` if `filename` or its stored metadata do not exist.
    """
    if cache_dir is None:
        cache_dir = DATASET_CACHE

    cache_path = os.path.join(cache_dir, filename)
    if not os.path.exists(cache_path):
        raise FileNotFoundError(u"file {} not found".format(cache_path))

    meta_path = cache_path + u'.json'
    if not os.path.exists(meta_path):
        raise FileNotFoundError(u"file {} not found".format(meta_path))

    with open(meta_path) as meta_file:
        metadata = json.load(meta_file)
    url = metadata[u'url']
    etag = metadata[u'etag']

    return url, etag


def cached_path(url_or_filename                  , cache_dir      = None)       :
    u"""
    Given something that might be a URL (or might be a local path),
    determine which. If it's a URL, download the file and cache it, and
    return the path to the cached file. If it's already a local path,
    make sure the file exists and then return the path.
    """
    if cache_dir is None:
        cache_dir = DATASET_CACHE
    url_or_filename = str(url_or_filename)

    parsed = urlparse(url_or_filename)

    if parsed.scheme in (u'http', u'https', u's3'):
        # URL, so get it from the cache (downloading if necessary)
        return get_from_cache(url_or_filename, cache_dir)
    elif os.path.exists(url_or_filename):
        # File, and it exists.
        return url_or_filename
    elif parsed.scheme == u'':
        # File, but it doesn't exist.
        raise FileNotFoundError(u"file {} not found".format(url_or_filename))
    else:
        # Something unknown
        raise ValueError(u"unable to parse {} as a URL or as a local path".format(url_or_filename))


def split_s3_path(url     )                   :
    u"""Split a full s3 path into the bucket name and path."""
    parsed = urlparse(url)
    if not parsed.netloc or not parsed.path:
        raise ValueError(u"bad s3 path {}".format(url))
    bucket_name = parsed.netloc
    s3_path = parsed.path
    # Remove '/' at beginning of path.
    if s3_path.startswith(u"/"):
        s3_path = s3_path[1:]
    return bucket_name, s3_path


def s3_request(func          ):
    u"""
    Wrapper function for s3 requests in order to create more helpful error
    messages.
    """

    @wraps(func)
    def wrapper(url     , *args, **kwargs):
        try:
            return func(url, *args, **kwargs)
        except ClientError as exc:
            if int(exc.response[u"Error"][u"Code"]) == 404:
                raise FileNotFoundError(u"file {} not found".format(url))
            else:
                raise

    return wrapper


@s3_request
def s3_etag(url     )                 :
    u"""Check ETag on S3 object."""
    s3_resource = boto3.resource(u"s3")
    bucket_name, s3_path = split_s3_path(url)
    s3_object = s3_resource.Object(bucket_name, s3_path)
    return s3_object.e_tag


@s3_request
def s3_get(url     , temp_file    )        :
    u"""Pull a file directly from S3."""
    s3_resource = boto3.resource(u"s3")
    bucket_name, s3_path = split_s3_path(url)
    s3_resource.Bucket(bucket_name).download_fileobj(s3_path, temp_file)


def http_get(url     , temp_file    )        :
    req = requests.get(url, stream=True)
    content_length = req.headers.get(u'Content-Length')
    total = int(content_length) if content_length is not None else None
    progress = Tqdm.tqdm(unit=u"B", total=total)
    for chunk in req.iter_content(chunk_size=1024):
        if chunk: # filter out keep-alive new chunks
            progress.update(len(chunk))
            temp_file.write(chunk)
    progress.close()


# TODO(joelgrus): do we want to do checksums or anything like that?
def get_from_cache(url     , cache_dir      = None)       :
    u"""
    Given a URL, look for the corresponding dataset in the local cache.
    If it's not there, download it. Then return the path to the cached file.
    """
    if cache_dir is None:
        cache_dir = DATASET_CACHE

    os.makedirs(cache_dir)

    # Get eTag to add to filename, if it exists.
    if url.startswith(u"s3://"):
        etag = s3_etag(url)
    else:
        response = requests.head(url, allow_redirects=True)
        if response.status_code != 200:
            raise IOError(u"HEAD request failed for url {} with status code {}"
                          .format(url, response.status_code))
        etag = response.headers.get(u"ETag")

    filename = url_to_filename(url, etag)

    # get cache path to put the file
    cache_path = os.path.join(cache_dir, filename)

    if not os.path.exists(cache_path):
        # Download to temporary file, then copy to cache dir once finished.
        # Otherwise you get corrupt cache entries if the download gets interrupted.
        with tempfile.NamedTemporaryFile() as temp_file:
            logger.info(u"%s not found in cache, downloading to %s", url, temp_file.name)

            # GET file object
            if url.startswith(u"s3://"):
                s3_get(url, temp_file)
            else:
                http_get(url, temp_file)

            # we are copying the file before closing it, so flush to avoid truncation
            temp_file.flush()
            # shutil.copyfileobj() starts at the current position, so go to the start
            temp_file.seek(0)

            logger.info(u"copying %s to cache at %s", temp_file.name, cache_path)
            with open(cache_path, u'wb') as cache_file:
                shutil.copyfileobj(temp_file, cache_file)

            logger.info(u"creating metadata file for %s", cache_path)
            meta = {u'url': url, u'etag': etag}
            meta_path = cache_path + u'.json'
            with open(meta_path, u'w') as meta_file:
                json.dump(meta, meta_file)

            logger.info(u"removing temp file %s", temp_file.name)

    return cache_path


def get_file_extension(path     , dot=True, lower       = True):
    ext = os.path.splitext(path)[1]
    ext = ext if dot else ext[1:]
    return ext.lower() if lower else ext




from __future__ import absolute_import
#overrides
import torch

from allennlp.modules.seq2seq_encoders.seq2seq_encoder import Seq2SeqEncoder

class PassThroughEncoder(Seq2SeqEncoder):
    u"""
    This class allows you to specify skipping a ``Seq2SeqEncoder`` just
    by changing a configuration file. This is useful for ablations and
    measuring the impact of different elements of your model.
    """
    def __init__(self, input_dim     )        :
        super(PassThroughEncoder, self).__init__()
        self._input_dim = input_dim

    #overrides
    def get_input_dim(self)       :
        return self._input_dim

    #overrides
    def get_output_dim(self)       :
        return self._input_dim

    #overrides
    def is_bidirectional(self):
        return False

    #overrides
    def forward(self,  # pylint: disable=arguments-differ
                inputs              ,
                mask                   = None)                     :
        # pylint: disable=unused-argument

        return inputs

PassThroughEncoder = Seq2SeqEncoder.register(u"pass_through")(PassThroughEncoder)




from __future__ import absolute_import
import logging
import collections
#typing

#overrides

from allennlp.common.file_utils import cached_path
from allennlp.data.dataset_readers.dataset_reader import DatasetReader
from allennlp.data.fields import Field, ListField, TextField, SpanField, MetadataField, SequenceLabelField
from allennlp.data.instance import Instance
from allennlp.data.tokenizers import Token
from allennlp.data.token_indexers import SingleIdTokenIndexer, TokenIndexer
from allennlp.data.dataset_readers.dataset_utils import  enumerate_spans
from io import open

logger = logging.getLogger(__name__)  # pylint: disable=invalid-name


class WinobiasReader(DatasetReader):
    u"""
    TODO(Mark): Add paper reference.

    Winobias is a dataset to analyse the issue of gender bias in co-reference
    resolution. It contains simple sentences with pro/anti stereotypical gender
    associations with which to measure the bias of a coreference system trained
    on another corpus. It is effectively a toy dataset and as such, uses very
    simplistic language; it has little use outside of evaluating a model for bias.

    The dataset is formatted with a single sentence per line, with a maximum of 2
    non-nested coreference clusters annotated using either square or round brackets.
    For example:

    [The salesperson] sold (some books) to the librarian because [she] was trying to sell (them).


    Returns a list of ``Instances`` which have four fields: ``text``, a ``TextField``
    containing the full sentence text, ``spans``, a ``ListField[SpanField]`` of inclusive start and
    end indices for span candidates, and ``metadata``, a ``MetadataField`` that stores the instance's
    original text. For data with gold cluster labels, we also include the original ``clusters``
    (a list of list of index pairs) and a ``SequenceLabelField`` of cluster ids for every span
    candidate in the ``metadata`` also.

    Parameters
    ----------
    max_span_width: ``int``, required.
        The maximum width of candidate spans to consider.
    token_indexers : ``Dict[str, TokenIndexer]``, optional
        This is used to index the words in the sentence.  See :class:`TokenIndexer`.
        Default is ``{"tokens": SingleIdTokenIndexer()}``.
    """
    def __init__(self,
                 max_span_width     ,
                 token_indexers                          = None,
                 lazy       = False)        :
        super(WinobiasReader, self).__init__(lazy)
        self._max_span_width = max_span_width
        self._token_indexers = token_indexers or {u"tokens": SingleIdTokenIndexer()}

    #overrides
    def _read(self, file_path     ):

        for sentence in open(cached_path(file_path), u"r"):
            tokens = sentence.strip().split(u" ")
            clusters                                          = collections.defaultdict(list)
            words = []
            for index, token in enumerate(tokens):
                # Coreference is annotated using [square brackets]
                # or (round brackets) around coreferent phrases.
                if u"[" in token and u"]" in token:
                    clusters[0].append((index, index))
                elif u"[" in token:
                    clusters[0].append((index, index))
                elif u"]" in token:
                    old_span = clusters[0][-1]
                    clusters[0][-1] = (old_span[0], index)

                if u"(" in token and u")" in token:
                    clusters[1].append((index, index))
                elif u"(" in token:
                    clusters[1].append((index, index))
                elif u")" in token:
                    old_span = clusters[1][-1]
                    clusters[1][-1] = (old_span[0], index)

                if token.endswith(u"."):
                    # Winobias is tokenised, but not for full stops.
                    # We'll just special case them here.
                    token = token[:-1]
                    words.append(token.strip(u"[]()"))
                    words.append(u".")
                else:
                    words.append(token.strip(u"[]()"))

            yield self.text_to_instance([Token(x) for x in words], [x for x in list(clusters.values())])

    #overrides
    def text_to_instance(self,  # type: ignore
                         sentence             ,
                         gold_clusters                                        = None)            :
        # pylint: disable=arguments-differ
        u"""
        Parameters
        ----------
        sentences : ``List[Token]``, required.
            The already tokenised sentence to analyse.
        gold_clusters : ``Optional[List[List[Tuple[int, int]]]]``, optional (default = None)
            A list of all clusters in the sentence, represented as word spans. Each cluster
            contains some number of spans, which can be nested and overlap, but will never
            exactly match between clusters.

        Returns
        -------
        An ``Instance`` containing the following ``Fields``:
            text : ``TextField``
                The text of the full sentence.
            spans : ``ListField[SpanField]``
                A ListField containing the spans represented as ``SpanFields``
                with respect to the sentence text.
            span_labels : ``SequenceLabelField``, optional
                The id of the cluster which each possible span belongs to, or -1 if it does
                 not belong to a cluster. As these labels have variable length (it depends on
                 how many spans we are considering), we represent this a as a ``SequenceLabelField``
                 with respect to the ``spans ``ListField``.
        """
        metadata                 = {u"original_text": sentence}
        if gold_clusters is not None:
            metadata[u"clusters"] = gold_clusters

        text_field = TextField(sentence, self._token_indexers)

        cluster_dict = {}
        if gold_clusters is not None:
            for cluster_id, cluster in enumerate(gold_clusters):
                for mention in cluster:
                    cluster_dict[tuple(mention)] = cluster_id

        spans              = []
        span_labels                      = [] if gold_clusters is not None else None

        for start, end in enumerate_spans(sentence, max_span_width=self._max_span_width):
            if span_labels is not None:
                if (start, end) in cluster_dict:
                    span_labels.append(cluster_dict[(start, end)])
                else:
                    span_labels.append(-1)

            spans.append(SpanField(start, end, text_field))

        span_field = ListField(spans)
        metadata_field = MetadataField(metadata)

        fields                   = {u"text": text_field,
                                    u"spans": span_field,
                                    u"metadata": metadata_field}
        if span_labels is not None:
            fields[u"span_labels"] = SequenceLabelField(span_labels, span_field)

        return Instance(fields)

WinobiasReader = DatasetReader.register(u"winobias")(WinobiasReader)



# pylint: disable=no-self-use,invalid-name



from __future__ import with_statement
from __future__ import division
from __future__ import absolute_import
import json

from allennlp.common.testing import AllenNlpTestCase
from allennlp.semparse.worlds.world import ExecutionError
from allennlp.semparse.worlds.nlvr_world import NlvrWorld
from io import open


class TestNlvrWorld(AllenNlpTestCase):
    def setUp(self):
        super(TestNlvrWorld, self).setUp()
        test_filename = self.FIXTURES_ROOT / u"data" / u"nlvr" / u"sample_ungrouped_data.jsonl"
        data = [json.loads(line)[u"structured_rep"] for line in open(test_filename).readlines()]
        self.worlds = [NlvrWorld(rep) for rep in data]
        # y_loc increases as we go down from top to bottom, and x_loc from left to right. That is,
        # the origin is at the top-left corner.
        custom_rep = [[{u"y_loc": 79, u"size": 20, u"type": u"triangle", u"x_loc": 27, u"color": u"Yellow"},
                       {u"y_loc": 55, u"size": 10, u"type": u"circle", u"x_loc": 47, u"color": u"Black"}],
                      [{u"y_loc": 44, u"size": 30, u"type": u"square", u"x_loc": 10, u"color": u"#0099ff"},
                       {u"y_loc": 74, u"size": 30, u"type": u"square", u"x_loc": 40, u"color": u"Yellow"}],
                      [{u"y_loc": 60, u"size": 10, u"type": u"triangle", u"x_loc": 12, u"color": u"#0099ff"}]]
        self.custom_world = NlvrWorld(custom_rep)

    def test_get_action_sequence_removes_currying_for_all_nlvr_functions(self):
        world = self.worlds[0]
        # box_color_filter
        logical_form = u"(member_color_all_equals all_boxes color_blue)"
        parsed_logical_form = world.parse_logical_form(logical_form)
        action_sequence = world.get_action_sequence(parsed_logical_form)
        assert u'b -> [<b,<c,b>>, b, c]' in action_sequence

        # box_shape_filter
        logical_form = u"(member_shape_all_equals all_boxes shape_square)"
        parsed_logical_form = world.parse_logical_form(logical_form)
        action_sequence = world.get_action_sequence(parsed_logical_form)
        assert u'b -> [<b,<s,b>>, b, s]' in action_sequence

        # box_count_filter
        logical_form = u"(member_count_equals all_boxes 3)"
        parsed_logical_form = world.parse_logical_form(logical_form)
        action_sequence = world.get_action_sequence(parsed_logical_form)
        assert u'b -> [<b,<e,b>>, b, e]' in action_sequence

        # assert_color
        logical_form = u"(object_color_all_equals all_objects color_blue)"
        parsed_logical_form = world.parse_logical_form(logical_form)
        action_sequence = world.get_action_sequence(parsed_logical_form)
        assert u't -> [<o,<c,t>>, o, c]' in action_sequence

        # assert_shape
        logical_form = u"(object_shape_all_equals all_objects shape_square)"
        parsed_logical_form = world.parse_logical_form(logical_form)
        action_sequence = world.get_action_sequence(parsed_logical_form)
        assert u't -> [<o,<s,t>>, o, s]' in action_sequence

        # assert_box_count
        logical_form = u"(box_count_equals all_boxes 1)"
        parsed_logical_form = world.parse_logical_form(logical_form)
        action_sequence = world.get_action_sequence(parsed_logical_form)
        assert u't -> [<b,<e,t>>, b, e]' in action_sequence

        # assert_object_count
        logical_form = u"(object_count_equals all_objects 1)"
        parsed_logical_form = world.parse_logical_form(logical_form)
        action_sequence = world.get_action_sequence(parsed_logical_form)
        assert u't -> [<o,<e,t>>, o, e]' in action_sequence

    def test_logical_form_with_assert_executes_correctly(self):
        nlvr_world = self.worlds[0]
        # Utterance is "There is a circle closely touching a corner of a box." and label is "True".
        logical_form_true = u"(object_count_greater_equals (touch_corner (circle (all_objects))) 1)"
        assert nlvr_world.execute(logical_form_true) is True
        logical_form_false = u"(object_count_equals (touch_corner (circle (all_objects))) 0)"
        assert nlvr_world.execute(logical_form_false) is False

    def test_logical_form_with_box_filter_executes_correctly(self):
        nlvr_world = self.worlds[2]
        # Utterance is "There is a box without a blue item." and label is "False".
        logical_form = u"(box_exists (member_color_none_equals all_boxes color_blue))"
        assert nlvr_world.execute(logical_form) is False

    def test_logical_form_with_box_filter_within_object_filter_executes_correctly(self):
        nlvr_world = self.worlds[2]
        # Utterance is "There are at least three blue items in boxes with blue items" and label
        # is "True".
        logical_form = u"(object_count_greater_equals \
                            (object_in_box (member_color_any_equals all_boxes color_blue)) 3)"
        assert nlvr_world.execute(logical_form) is True

    def test_logical_form_with_same_color_executes_correctly(self):
        nlvr_world = self.worlds[1]
        # Utterance is "There are exactly two blocks of the same color." and label is "True".
        logical_form = u"(object_count_equals (same_color all_objects) 2)"
        assert nlvr_world.execute(logical_form) is True

    def test_logical_form_with_same_shape_executes_correctly(self):
        nlvr_world = self.worlds[0]
        # Utterance is "There are less than three black objects of the same shape" and label is "False".
        logical_form = u"(object_count_lesser (same_shape (black (all_objects))) 3)"
        assert nlvr_world.execute(logical_form) is False

    def test_logical_form_with_touch_wall_executes_correctly(self):
        nlvr_world = self.worlds[0]
        # Utterance is "There are two black circles touching a wall" and label is "False".
        logical_form = u"(object_count_greater_equals (touch_wall (black (circle (all_objects)))) 2)"
        assert nlvr_world.execute(logical_form) is False

    def test_logical_form_with_not_executes_correctly(self):
        nlvr_world = self.worlds[2]
        # Utterance is "There are at most two medium triangles not touching a wall." and label is "True".
        logical_form = (u"(object_count_lesser_equals ((negate_filter touch_wall) \
                                                                     (medium (triangle (all_objects)))) 2)")
        assert nlvr_world.execute(logical_form) is True

    def test_logical_form_with_color_comparison_executes_correctly(self):
        nlvr_world = self.worlds[0]
        # Utterance is "The color of the circle touching the wall is black." and label is "True".
        logical_form = u"(object_color_all_equals (circle (touch_wall (all_objects))) color_black)"
        assert nlvr_world.execute(logical_form) is True

    def test_logical_form_with_object_filter_returns_correct_action_sequence(self):
        nlvr_world = self.worlds[0]
        logical_form = u"(object_color_all_equals (circle (touch_wall (all_objects))) color_black)"
        expression = nlvr_world.parse_logical_form(logical_form)
        action_sequence = nlvr_world.get_action_sequence(expression)
        assert action_sequence == [u'@start@ -> t', u't -> [<o,<c,t>>, o, c]',
                                   u'<o,<c,t>> -> object_color_all_equals', u'o -> [<o,o>, o]',
                                   u'<o,o> -> circle', u'o -> [<o,o>, o]', u'<o,o> -> touch_wall',
                                   u'o -> all_objects', u'c -> color_black']

    def test_logical_form_with_box_filter_returns_correct_action_sequence(self):
        nlvr_world = self.worlds[0]
        logical_form = u"(box_exists (member_color_none_equals all_boxes color_blue))"
        expression = nlvr_world.parse_logical_form(logical_form)
        action_sequence = nlvr_world.get_action_sequence(expression)
        assert action_sequence == [u'@start@ -> t', u't -> [<b,t>, b]', u'<b,t> -> box_exists',
                                   u'b -> [<b,<c,b>>, b, c]', u'<b,<c,b>> -> member_color_none_equals',
                                   u'b -> all_boxes', u'c -> color_blue']

    def test_spatial_relations_return_objects_in_the_same_box(self):
        # "above", "below", "top", "bottom" are relations defined only for objects within the same
        # box. So they should not return objects from other boxes.
        world = self.custom_world
        # Asserting that the color of the objects above the yellow triangle is only black (it is not
        # yellow or blue, which are colors of objects from other boxes)
        assert world.execute(u"(object_color_all_equals (above (yellow (triangle all_objects)))"
                             u" color_black)") is True
        # Asserting that the only shape below the blue square is a square.
        assert world.execute(u"(object_shape_all_equals (below (blue (square all_objects)))"
                             u" shape_square)") is True
        # Asserting the shape of the object at the bottom in the box with a circle is triangle.
        logical_form = (u"(object_shape_all_equals (bottom (object_in_box"
                        u" (member_shape_any_equals all_boxes shape_circle))) shape_triangle)")
        assert world.execute(logical_form) is True

        # Asserting the shape of the object at the top of the box with all squares is a square (!).
        logical_form = (u"(object_shape_all_equals (top (object_in_box"
                        u" (member_shape_all_equals all_boxes shape_square))) shape_square)")
        assert world.execute(logical_form) is True

    def test_touch_object_executes_correctly(self):
        world = self.custom_world
        # Assert that there is a yellow square touching a blue square.
        assert world.execute(u"(object_exists (yellow (square (touch_object (blue "
                             u"(square all_objects))))))") is True
        # Assert that the triangle does not touch the circle (they are out of vertical range).
        assert world.execute(u"(object_shape_none_equals (touch_object (triangle all_objects))"
                             u" shape_circle)") is True

    def test_spatial_relations_with_objects_from_different_boxes(self):
        # When the objects are from different boxes, top and bottom should return objects from
        # respective boxes.
        world = self.custom_world
        # There are triangles in two boxes, so top should return the top objects from both boxes.
        assert world.execute(u"(object_count_equals (top (object_in_box (member_shape_any_equals "
                             u"all_boxes shape_triangle))) 2)") is True

    def test_count_with_all_equals_throws_execution_error(self):
        # "*_all_equals" is a comparison valid only for sets (of colors and shapes). A comparison
        # with count should use "*_equals" instead.
        world = self.custom_world
        with self.assertRaises(ExecutionError):
            world.execute(u"(object_count_all_equals (top (object_in_box (member_shape_any_equals "
                          u"all_boxes shape_triangle))) 2)")

    def test_shape_with_equals_throws_execution_error(self):
        # "*_all_equals" is a comparison valid only for counts. A comparison with sets should use
        # "*_equals", "*_any_equals" or "*_non_equals" instead.
        world = self.custom_world
        with self.assertRaises(ExecutionError):
            world.execute(u"(object_shape_equals (top (object_in_box (member_shape_any_equals "
                          u"all_boxes shape_triangle))) shape_triangle)")

    def test_same_and_different_execute_correctly(self):
        world = self.custom_world
        # All the objects in the box with two objects of the same shape are squares.
        assert world.execute(u"(object_shape_all_equals "
                             u"(object_in_box (member_shape_same (member_count_equals all_boxes 2)))"
                             u" shape_square)") is True
        # There is a circle in the box with objects of different shapes.
        assert world.execute(u"(object_shape_any_equals (object_in_box "
                             u"(member_shape_different all_boxes)) shape_circle)") is True

    def test_get_agenda_for_sentence(self):
        world = self.worlds[0]
        agenda = world.get_agenda_for_sentence(u"there is a tower with exactly two yellow blocks")
        assert set(agenda) == set([u'c -> color_yellow', u'<b,t> -> box_exists', u'e -> 2'])
        agenda = world.get_agenda_for_sentence(u"There is at most one yellow item closely touching "
                                               u"the bottom of a box.")
        assert set(agenda) == set([u'<o,o> -> yellow', u'<o,o> -> touch_bottom', u'e -> 1'])
        agenda = world.get_agenda_for_sentence(u"There is at most one yellow item closely touching "
                                               u"the right wall of a box.")
        assert set(agenda) == set([u'<o,o> -> yellow', u'<o,o> -> touch_right', u'e -> 1'])
        agenda = world.get_agenda_for_sentence(u"There is at most one yellow item closely touching "
                                               u"the left wall of a box.")
        assert set(agenda) == set([u'<o,o> -> yellow', u'<o,o> -> touch_left', u'e -> 1'])
        agenda = world.get_agenda_for_sentence(u"There is at most one yellow item closely touching "
                                               u"a wall of a box.")
        assert set(agenda) == set([u'<o,o> -> yellow', u'<o,o> -> touch_wall', u'e -> 1'])
        agenda = world.get_agenda_for_sentence(u"There is exactly one square touching any edge")
        assert set(agenda) == set([u'<o,o> -> square', u'<o,o> -> touch_wall', u'e -> 1'])
        agenda = world.get_agenda_for_sentence(u"There is exactly one square not touching any edge")
        assert set(agenda) == set([u'<o,o> -> square', u'<o,o> -> touch_wall', u'e -> 1',
                                   u'<<o,o>,<o,o>> -> negate_filter'])
        agenda = world.get_agenda_for_sentence(u"There is only 1 tower with 1 blue block at the base")
        assert set(agenda) == set([u'<o,o> -> blue', u'e -> 1', u'<o,o> -> bottom', u'e -> 1'])
        agenda = world.get_agenda_for_sentence(u"There is only 1 tower that has 1 blue block at the top")
        assert set(agenda) == set([u'<o,o> -> blue', u'e -> 1', u'<o,o> -> top', u'e -> 1',
                                   u'b -> all_boxes'])
        agenda = world.get_agenda_for_sentence(u"There is exactly one square touching the blue "
                                               u"triangle")
        assert set(agenda) == set([u'<o,o> -> square', u'<o,o> -> blue', u'<o,o> -> triangle',
                                   u'<o,o> -> touch_object', u'e -> 1'])

    def test_get_agenda_for_sentence_correctly_adds_object_filters(self):
        # In logical forms that contain "box_exists" at the top, there can never be object filtering
        # operations like "blue", "square" etc. In those cases, strings like "blue" and "square" in
        # sentences should map to "color_blue" and "shape_square" respectively.
        world = self.worlds[0]
        agenda = world.get_agenda_for_sentence(u"there is a box with exactly two yellow triangles "
                                               u"touching the top edge")
        assert u"<o,o> -> yellow" not in agenda
        assert u"c -> color_yellow" in agenda
        assert u"<o,o> -> triangle" not in agenda
        assert u"s -> shape_triangle" in agenda
        assert u"<o,o> -> touch_top" not in agenda
        agenda = world.get_agenda_for_sentence(u"there are exactly two yellow triangles touching the"
                                               u" top edge")
        assert u"<o,o> -> yellow" in agenda
        assert u"c -> color_yellow" not in agenda
        assert u"<o,o> -> triangle" in agenda
        assert u"s -> shape_triangle" not in agenda
        assert u"<o,o> -> touch_top" in agenda



# pylint: disable=no-self-use,invalid-name,abstract-method

from __future__ import absolute_import
#typing

from allennlp.data.fields import TextField
from allennlp.data.instance import Instance
from allennlp.data.dataset_readers import DatasetReader
from allennlp.data.token_indexers import SingleIdTokenIndexer
from allennlp.data.tokenizers import Token
from allennlp.common.testing import AllenNlpTestCase
from allennlp.common.util import ensure_list


class LazyDatasetReader(DatasetReader):
    def __init__(self, instances                , lazy      )        :
        super(LazyDatasetReader, self).__init__()
        self.lazy = lazy
        self._instances = instances
        self.num_reads = 0

    def _read(self, _     )                      :
        self.num_reads += 1
        return (instance for instance in self._instances)


class TestLazyDatasetReader(AllenNlpTestCase):
    def setUp(self):
        super(TestLazyDatasetReader, self).setUp()
        token_indexer = {u"tokens": SingleIdTokenIndexer()}

        field1 = TextField([Token(t) for t in [u"this", u"is", u"a", u"sentence", u"."]],
                           token_indexer)
        field2 = TextField([Token(t) for t in [u"this", u"is", u"a", u"different", u"sentence", u"."]],
                           token_indexer)
        field3 = TextField([Token(t) for t in [u"here", u"is", u"a", u"sentence", u"."]],
                           token_indexer)
        field4 = TextField([Token(t) for t in [u"this", u"is", u"short"]],
                           token_indexer)
        self.instances = [Instance({u"text1": field1, u"text2": field2}),
                          Instance({u"text1": field3, u"text2": field4})]

    def test_lazy(self):
        reader = LazyDatasetReader(self.instances, lazy=True)
        assert reader.num_reads == 0

        instances = reader.read(u'path/to/file')

        for _ in range(10):
            _instances = (i for i in instances)
            assert ensure_list(_instances) == self.instances

        assert reader.num_reads == 10

    def test_non_lazy(self):
        reader = LazyDatasetReader(self.instances, lazy=False)
        assert reader.num_reads == 0

        instances = reader.read(u'path/to/file')

        for _ in range(10):
            _instances = (i for i in instances)
            assert ensure_list(_instances) == self.instances

        assert reader.num_reads == 1



# pylint: disable=no-self-use,invalid-name


from __future__ import division
from __future__ import absolute_import
import pytest

from allennlp.data.dataset_readers.semantic_role_labeling import SrlReader
from allennlp.common.util import ensure_list
from allennlp.common.testing import AllenNlpTestCase

class TestSrlReader(object):
    @pytest.mark.parametrize(u"lazy", (True, False))
    def test_read_from_file(self, lazy):
        conll_reader = SrlReader(lazy=lazy)
        instances = conll_reader.read(AllenNlpTestCase.FIXTURES_ROOT / u'conll_2012' / u'subdomain')
        instances = ensure_list(instances)

        fields = instances[0].fields
        tokens = [t.text for t in fields[u'tokens'].tokens]
        assert tokens == [u"Mali", u"government", u"officials", u"say", u"the", u"woman", u"'s",
                          u"confession", u"was", u"forced", u"."]
        assert fields[u"verb_indicator"].labels[3] == 1
        assert fields[u"tags"].labels == [u'B-ARG0', u'I-ARG0', u'I-ARG0', u'B-V', u'B-ARG1',
                                         u'I-ARG1', u'I-ARG1', u'I-ARG1', u'I-ARG1', u'I-ARG1', u'O']

        fields = instances[1].fields
        tokens = [t.text for t in fields[u'tokens'].tokens]
        assert tokens == [u"Mali", u"government", u"officials", u"say", u"the", u"woman", u"'s",
                          u"confession", u"was", u"forced", u"."]
        assert fields[u"verb_indicator"].labels[8] == 1
        assert fields[u"tags"].labels == [u'O', u'O', u'O', u'O', u'B-ARG1', u'I-ARG1',
                                         u'I-ARG1', u'I-ARG1', u'B-V', u'B-ARG2', u'O']

        fields = instances[2].fields
        tokens = [t.text for t in fields[u'tokens'].tokens]
        assert tokens == [u'The', u'prosecution', u'rested', u'its', u'case', u'last', u'month', u'after',
                          u'four', u'months', u'of', u'hearings', u'.']
        assert fields[u"verb_indicator"].labels[2] == 1
        assert fields[u"tags"].labels == [u'B-ARG0', u'I-ARG0', u'B-V', u'B-ARG1', u'I-ARG1', u'B-ARGM-TMP',
                                         u'I-ARGM-TMP', u'B-ARGM-TMP', u'I-ARGM-TMP', u'I-ARGM-TMP',
                                         u'I-ARGM-TMP', u'I-ARGM-TMP', u'O']

        fields = instances[3].fields
        tokens = [t.text for t in fields[u'tokens'].tokens]
        assert tokens == [u'The', u'prosecution', u'rested', u'its', u'case', u'last', u'month', u'after',
                          u'four', u'months', u'of', u'hearings', u'.']
        assert fields[u"verb_indicator"].labels[11] == 1
        assert fields[u"tags"].labels == [u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'O', u'B-V', u'O']

        # Tests a sentence with no verbal predicates.
        fields = instances[4].fields
        tokens = [t.text for t in fields[u'tokens'].tokens]
        assert tokens == [u"Denise", u"Dillon", u"Headline", u"News", u"."]
        assert fields[u"verb_indicator"].labels == [0, 0, 0, 0, 0]
        assert fields[u"tags"].labels == [u'O', u'O', u'O', u'O', u'O']

    def test_srl_reader_can_filter_by_domain(self):

        conll_reader = SrlReader(domain_identifier=u"subdomain2")
        instances = conll_reader.read(AllenNlpTestCase.FIXTURES_ROOT / u'conll_2012')
        instances = ensure_list(instances)
        # If we'd included the folder, we'd have 9 instances.
        assert len(instances) == 2




from __future__ import absolute_import
from copy import deepcopy
#typing
import numpy

from parsimonious.grammar import Grammar

from allennlp.semparse.contexts.atis_tables import * # pylint: disable=wildcard-import,unused-wildcard-import
from allennlp.semparse.contexts.sql_table_context import\
        SqlTableContext, SqlVisitor, generate_one_of_string, format_action

from allennlp.data.tokenizers import Token, WordTokenizer
try:
    from itertools import izip
except:
    izip = zip


def get_strings_from_utterance(tokenized_utterance             )                        :
    u"""
    Based on the current utterance, return a dictionary where the keys are the strings in the utterance
    that map to lists of the token indices that they are linked to.
    """
    string_linking_scores                       = defaultdict(list)
    for index, (first_token, second_token) in enumerate(izip(tokenized_utterance, tokenized_utterance[1:])):
        for string in ATIS_TRIGGER_DICT.get(first_token.text.lower(), []):
            string_linking_scores[string].append(index)

        bigram = "{first_token.text} {second_token.text}".lower()
        for string in ATIS_TRIGGER_DICT.get(bigram, []):
            string_linking_scores[string].extend([index, index + 1])

    if tokenized_utterance[-1].text.lower() in ATIS_TRIGGER_DICT:
        for string in ATIS_TRIGGER_DICT[tokenized_utterance[-1].text.lower()]:
            string_linking_scores[string].append(len(tokenized_utterance)-1)

    date = get_date_from_utterance(tokenized_utterance)
    if date:
        for day in DAY_OF_WEEK_INDEX[date.weekday()]:
            string_linking_scores[day] = []

    return string_linking_scores

class AtisWorld(object):
    u"""
    World representation for the Atis SQL domain. This class has a ``SqlTableContext`` which holds the base
    grammars, it then augments this grammar with the entities that are detected from utterances.

    Parameters
    ----------
    utterances: ``List[str]``
        A list of utterances in the interaction, the last element in this list is the
        current utterance that we are interested in.
    """
    sql_table_context = SqlTableContext(TABLES)

    def __init__(self, utterances           , tokenizer=None)        :
        self.utterances =  utterances
        self.tokenizer = tokenizer if tokenizer else WordTokenizer()
        self.tokenized_utterances = [self.tokenizer.tokenize(utterance) for utterance in self.utterances]
        valid_actions, linking_scores = self.init_all_valid_actions()
        self.valid_actions =  valid_actions

        # This has shape (num_entities, num_utterance_tokens).
        self.linking_scores: numpy.ndarray = linking_scores
        self.grammar_str: unicode = self.get_grammar_str()
        self.grammar_with_context: Grammar = Grammar(self.grammar_str)

    def get_valid_actions(self)                        :
        return self.valid_actions

    def init_all_valid_actions(self)                                              :
        u"""
        We initialize the valid actions with the global actions. We then iterate through the
        utterances up to and including the current utterance and add the valid strings.
        """

        valid_actions = deepcopy(self.sql_table_context.valid_actions)
        linking_scores = []
        current_tokenized_utterance = [] if not self.tokenized_utterances\
                else self.tokenized_utterances[-1]

        strings           = set()
        for tokenized_utterance in self.tokenized_utterances:
            string_linking_dict = get_strings_from_utterance(tokenized_utterance)
            strings.update(list(string_linking_dict.keys()))

        # We want to sort things in reverse here to be consistent with the grammar.
        # The parser is greedy which means that if we have a rule that has
        # multiple options for the right hand side, the first one that succeeds is
        # the one that is used. For example, if ``1400`` appears in the query, and
        # both ``1400`` and ``1`` are valid numbers, then we want to try to match
        # ``1400`` first. Otherwise, ``1`` will succeed but nothing will match ``400``.
        # The same applies for strings here.
        strings_list            = sorted(strings, reverse=True)

        # We construct the linking scores for strings from the ``string_linking_dict`` here.
        string_linking_scores = []
        for string in strings_list:
            entity_linking = [0 for token in current_tokenized_utterance]
            # string_linking_dict has the strings and linking scores from the last utterance.
            # If the string is not in the last utterance, then the linking scores will be all 0.
            for token_index in string_linking_dict.get(string, []):
                entity_linking[token_index] = 1
            string_linking_scores.append(entity_linking)
        linking_scores.extend(string_linking_scores)

        for string in strings_list:
            action = format_action(u'string', string)
            if action not in valid_actions[u'string']:
                valid_actions[u'string'].append(action)

        numbers = set([u'0', u'1'])
        number_linking_dict                       = {}

        for utterance, tokenized_utterance in izip(self.utterances, self.tokenized_utterances):
            number_linking_dict = get_numbers_from_utterance(utterance, tokenized_utterance)
            numbers.update(list(number_linking_dict.keys()))
        numbers_list            = sorted(numbers, reverse=True)

        # We construct the linking scores for numbers from the ``number_linking_dict`` here.
        number_linking_scores = []
        for number in numbers_list:
            entity_linking = [0 for token in current_tokenized_utterance]
            # number_linking_scores has the numbers and linking scores from the last utterance.
            # If the number is not in the last utterance, then the linking scores will be all 0.
            for token_index in number_linking_dict.get(number, []):
                entity_linking[token_index] = 1
            number_linking_scores.append(entity_linking)
        linking_scores.extend(number_linking_scores)

        for number in list(numbers_list):
            action = format_action(u'number', number)
            valid_actions[u'number'].append(action)
        return valid_actions, numpy.array(linking_scores)

    def get_grammar_str(self)       :
        u"""
        Generate a string that can be used to instantiate a ``Grammar`` object. The string is a sequence of
        rules that define the grammar.
        """
        grammar_str_with_context = self.sql_table_context.grammar_str
        numbers = [number.split(u" -> ")[1].lstrip(u'["').rstrip(u'"]') for\
                   number in sorted(self.valid_actions[u'number'], reverse=True)]
        strings = [string .split(u" -> ")[1].lstrip(u'["').rstrip(u'"]') for\
                   string in sorted(self.valid_actions[u'string'], reverse=True)]

        grammar_str_with_context += generate_one_of_string(u"number", numbers)
        grammar_str_with_context += generate_one_of_string(u"string", strings)
        return grammar_str_with_context


    def get_action_sequence(self, query     )             :
        sql_visitor = SqlVisitor(self.grammar_with_context)
        if query:
            action_sequence = sql_visitor.parse(query)
            return action_sequence
        return []

    def all_possible_actions(self)             :
        u"""
        Return a sorted list of strings representing all possible actions
        of the form: nonterminal -> [right_hand_side]
        """
        all_actions = set()
        for _, action_list in list(self.valid_actions.items()):
            for action in action_list:
                all_actions.add(action)
        return sorted(all_actions)

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            return all([self.valid_actions == other.valid_actions,
                        numpy.array_equal(self.linking_scores, other.linking_scores),
                        self.utterances == other.utterances,
                        self.grammar_str == other.grammar_str])
        return False



# pylint: disable=no-self-use,invalid-name


from __future__ import with_statement
from __future__ import absolute_import
import pytest

from allennlp.common import Params
from allennlp.common.checks import ConfigurationError
from allennlp.data.iterators import BucketIterator
from allennlp.tests.data.iterators.basic_iterator_test import IteratorTest


class TestBucketIterator(IteratorTest):
    # pylint: disable=protected-access
    def test_create_batches_groups_correctly(self):
        iterator = BucketIterator(batch_size=2, padding_noise=0, sorting_keys=[(u'text', u'num_tokens')])
        batches = list(iterator._create_batches(self.instances, shuffle=False))
        grouped_instances = [batch.instances for batch in batches]
        assert grouped_instances == [[self.instances[4], self.instances[2]],
                                     [self.instances[0], self.instances[1]],
                                     [self.instances[3]]]

    def test_create_batches_groups_correctly_with_max_instances(self):
        # If we knew all the instances, the correct order is 4 -> 2 -> 0 -> 1 -> 3.
        # Here max_instances_in_memory is 3, so we load instances [0, 1, 2]
        # and then bucket them by size into batches of size 2 to get [2, 0] -> [1].
        # Then we load the remaining instances and bucket them by size to get [4, 3].
        iterator = BucketIterator(batch_size=2,
                                  padding_noise=0,
                                  sorting_keys=[(u'text', u'num_tokens')],
                                  max_instances_in_memory=3)
        for test_instances in (self.instances, self.lazy_instances):
            batches = list(iterator._create_batches(test_instances, shuffle=False))
            grouped_instances = [batch.instances for batch in batches]
            assert grouped_instances == [[self.instances[2], self.instances[0]],
                                         [self.instances[1]],
                                         [self.instances[4], self.instances[3]]]

    def test_biggest_batch_first_works(self):
        iterator = BucketIterator(batch_size=2,
                                  padding_noise=0,
                                  sorting_keys=[(u'text', u'num_tokens')],
                                  biggest_batch_first=True)
        batches = list(iterator._create_batches(self.instances, shuffle=False))
        grouped_instances = [batch.instances for batch in batches]
        assert grouped_instances == [[self.instances[3]],
                                     [self.instances[0], self.instances[1]],
                                     [self.instances[4], self.instances[2]]]

    def test_from_params(self):
        # pylint: disable=protected-access
        params = Params({})

        with pytest.raises(ConfigurationError):
            iterator = BucketIterator.from_params(params)

        sorting_keys = [(u"s1", u"nt"), (u"s2", u"nt2")]
        params[u'sorting_keys'] = sorting_keys
        iterator = BucketIterator.from_params(params)

        assert iterator._sorting_keys == sorting_keys
        assert iterator._padding_noise == 0.1
        assert not iterator._biggest_batch_first
        assert iterator._batch_size == 32

        params = Params({
                u"sorting_keys": sorting_keys,
                u"padding_noise": 0.5,
                u"biggest_batch_first": True,
                u"batch_size": 100
        })

        iterator = BucketIterator.from_params(params)
        assert iterator._sorting_keys == sorting_keys
        assert iterator._padding_noise == 0.5
        assert iterator._biggest_batch_first
        assert iterator._batch_size == 100

    def test_bucket_iterator_maximum_samples_per_batch(self):
        iterator = BucketIterator(
                batch_size=3, padding_noise=0,
                sorting_keys=[(u'text', u'num_tokens')],
                maximum_samples_per_batch=[u'num_tokens', 9]
        )
        batches = list(iterator._create_batches(self.instances, shuffle=False))

        # ensure all instances are in a batch
        grouped_instances = [batch.instances for batch in batches]
        num_instances = sum(len(group) for group in grouped_instances)
        assert num_instances == len(self.instances)

        # ensure all batches are sufficiently small
        for batch in batches:
            batch_sequence_length = max(
                    [instance.get_padding_lengths()[u'text'][u'num_tokens']
                     for instance in batch.instances]
            )
            assert batch_sequence_length * len(batch.instances) <= 9



#!/usr/bin/env python
#
# See the accompanying LICENSE file.
#
import urllib2
import hashlib
import re

sqlitevers=(
    '3250200',
    '3250100',
    '3250000',
    '3240000',
    '3230100',
    '3230000',
    '3220000',
    '3210000',
    '3200100',
    '3200000',
    '3190300',
    '3190200',
    '3190100',
    '3190000',
    '3180000',
    '3170000',
    '3160200',
    '3160100',
    '3160000',
    '3150200',
    '3150100',
    '3150000',
    '3140100',
    '3140000',
    '3130000',
    )

# Checks the checksums file

def getline(url):
    for line in open("checksums", "rtU"):
        line=line.strip()
        if len(line)==0 or line[0]=="#":
            continue
        l=[l.strip() for l in line.split()]
        if len(l)!=4:
            print "Invalid line in checksums file:", line
            raise ValueError("Bad checksums file")
        if l[0]==url:
            return l[1:]
    return None

def check(url, data):
    d=["%s" % (len(data),), hashlib.sha1(data).hexdigest(), hashlib.md5(data).hexdigest()]
    line=getline(url)
    if line:
        if line!=d:
            print "Checksums mismatch for", url
            print "checksums file is", line
            print "Download is", d
    else:
        print url,
        if url.endswith(".zip"):
            print "  ",
        print d[0], d[1], d[2]

# They keep messing with where files are in URI - this code is also in setup.py
def fixup_download_url(url):
    ver=re.search("3[0-9]{6}", url)
    if ver:
        ver=int(ver.group(0))
        if ver>=3071600:
            if ver>=3220000:
                year="2018"
            elif ver>=3160000:
                year="2017"
            elif ver>=3100000:
                year="2016"
            elif ver>=3080800:
                year="2015"
            elif ver>=3080300:
                year="2014"
            else:
                year="2013"
            if "/"+year+"/" not in url:
                url=url.split("/")
                url.insert(3, year)
                return "/".join(url)
    return url

for v in sqlitevers:
    # Windows amalgamation
    AURL="https://sqlite.org/sqlite-amalgamation-%s.zip" % (v,)
    AURL=fixup_download_url(AURL)
    try:
        data=urllib2.urlopen(AURL).read()
    except:
        print AURL
        raise
    check(AURL, data)
    # All other platforms amalgamation
    AURL="https://sqlite.org/sqlite-autoconf-%s.tar.gz" % (v,)
    AURL=fixup_download_url(AURL)
    try:
        data=urllib2.urlopen(AURL).read()
    except:
        print AURL
        raise
    check(AURL, data)



# pylint: disable=no-self-use,invalid-name



from __future__ import division
from __future__ import with_statement
from __future__ import absolute_import
import pytest
import torch

from allennlp.common import Params
from allennlp.common.checks import ConfigurationError
from allennlp.data import Vocabulary
from allennlp.modules.text_field_embedders import BasicTextFieldEmbedder
from allennlp.common.testing import AllenNlpTestCase


class TestBasicTextFieldEmbedder(AllenNlpTestCase):
    def setUp(self):
        super(TestBasicTextFieldEmbedder, self).setUp()
        self.vocab = Vocabulary()
        self.vocab.add_token_to_namespace(u"1")
        self.vocab.add_token_to_namespace(u"2")
        self.vocab.add_token_to_namespace(u"3")
        self.vocab.add_token_to_namespace(u"4")
        params = Params({
                u"words1": {
                        u"type": u"embedding",
                        u"embedding_dim": 2
                        },
                u"words2": {
                        u"type": u"embedding",
                        u"embedding_dim": 5
                        },
                u"words3": {
                        u"type": u"embedding",
                        u"embedding_dim": 3
                        }
                })
        self.token_embedder = BasicTextFieldEmbedder.from_params(vocab=self.vocab, params=params)
        self.inputs = {
                u"words1": torch.LongTensor([[0, 2, 3, 5]]),
                u"words2": torch.LongTensor([[1, 4, 3, 2]]),
                u"words3": torch.LongTensor([[1, 5, 1, 2]])
                }

    def test_get_output_dim_aggregates_dimension_from_each_embedding(self):
        assert self.token_embedder.get_output_dim() == 10

    def test_forward_asserts_input_field_match(self):
        self.inputs[u'words4'] = self.inputs[u'words3']
        del self.inputs[u'words3']
        with pytest.raises(ConfigurationError):
            self.token_embedder(self.inputs)
        self.inputs[u'words3'] = self.inputs[u'words4']
        del self.inputs[u'words4']

    def test_forward_concats_resultant_embeddings(self):
        assert self.token_embedder(self.inputs).size() == (1, 4, 10)

    def test_forward_works_on_higher_order_input(self):
        params = Params({
                u"words": {
                        u"type": u"embedding",
                        u"num_embeddings": 20,
                        u"embedding_dim": 2,
                        },
                u"characters": {
                        u"type": u"character_encoding",
                        u"embedding": {
                                u"embedding_dim": 4,
                                u"num_embeddings": 15,
                                },
                        u"encoder": {
                                u"type": u"cnn",
                                u"embedding_dim": 4,
                                u"num_filters": 10,
                                u"ngram_filter_sizes": [3],
                                },
                        }
                })
        token_embedder = BasicTextFieldEmbedder.from_params(vocab=self.vocab, params=params)
        inputs = {
                u'words': (torch.rand(3, 4, 5, 6) * 20).long(),
                u'characters': (torch.rand(3, 4, 5, 6, 7) * 15).long(),
                }
        assert token_embedder(inputs, num_wrapping_dims=2).size() == (3, 4, 5, 6, 12)

    def test_forward_runs_with_non_bijective_mapping(self):
        elmo_fixtures_path = self.FIXTURES_ROOT / u'elmo'
        options_file = unicode(elmo_fixtures_path / u'options.json')
        weight_file = unicode(elmo_fixtures_path / u'lm_weights.hdf5')
        params = Params({
                u"words": {
                        u"type": u"embedding",
                        u"num_embeddings": 20,
                        u"embedding_dim": 2,
                        },
                u"elmo": {
                        u"type": u"elmo_token_embedder",
                        u"options_file": options_file,
                        u"weight_file": weight_file
                        },
                u"embedder_to_indexer_map": {u"words": [u"words"], u"elmo": [u"elmo", u"words"]}
                })
        token_embedder = BasicTextFieldEmbedder.from_params(self.vocab, params)
        inputs = {
                u'words': (torch.rand(3, 6) * 20).long(),
                u'elmo': (torch.rand(3, 6, 50) * 15).long(),
                }
        token_embedder(inputs)

    def test_old_from_params_new_from_params(self):

        old_params = Params({
                u"words1": {
                        u"type": u"embedding",
                        u"embedding_dim": 2
                        },
                u"words2": {
                        u"type": u"embedding",
                        u"embedding_dim": 5
                        },
                u"words3": {
                        u"type": u"embedding",
                        u"embedding_dim": 3
                        }
                })

        with pytest.warns(DeprecationWarning):
            BasicTextFieldEmbedder.from_params(params=old_params, vocab=self.vocab)

        new_params = Params({
                u"token_embedders": {
                        u"words1": {
                                u"type": u"embedding",
                                u"embedding_dim": 2
                                },
                        u"words2": {
                                u"type": u"embedding",
                                u"embedding_dim": 5
                                },
                        u"words3": {
                                u"type": u"embedding",
                                u"embedding_dim": 3
                                }
                        }
                })

        token_embedder = BasicTextFieldEmbedder.from_params(params=new_params, vocab=self.vocab)
        assert token_embedder(self.inputs).size() == (1, 4, 10)




from __future__ import absolute_import
#typing

#overrides
import torch

from allennlp.common.checks import check_dimensions_match
from allennlp.common.util import pad_sequence_to_length
from allennlp.data import Vocabulary
from allennlp.data.fields.production_rule_field import ProductionRuleArray
from allennlp.models.model import Model
from allennlp.models.semantic_parsing.wikitables.wikitables_decoder_state import WikiTablesDecoderState
from allennlp.modules import Embedding, Seq2SeqEncoder, Seq2VecEncoder, TextFieldEmbedder, TimeDistributed
from allennlp.modules.seq2vec_encoders import BagOfEmbeddingsEncoder
from allennlp.nn import util
from allennlp.nn.decoding import GrammarState, RnnState, ChecklistState
from allennlp.semparse.type_declarations import type_declaration
from allennlp.semparse.type_declarations.type_declaration import START_SYMBOL
from allennlp.semparse.worlds import WikiTablesWorld
from allennlp.training.metrics import Average, WikiTablesAccuracy
try:
    from itertools import izip
except:
    izip = zip



class WikiTablesSemanticParser(Model):
    u"""
    A ``WikiTablesSemanticParser`` is a :class:`Model` which takes as input a table and a question,
    and produces a logical form that answers the question when executed over the table.  The
    logical form is generated by a `type-constrained`, `transition-based` parser. This is an
    abstract class that defines most of the functionality related to the transition-based parser. It
    does not contain the implementation for actually training the parser. You may want to train it
    using a learning-to-search algorithm, in which case you will want to use
    ``WikiTablesErmSemanticParser``, or if you have a set of approximate logical forms that give the
    correct denotation, you will want to use ``WikiTablesMmlSemanticParser``.

    Parameters
    ----------
    vocab : ``Vocabulary``
    question_embedder : ``TextFieldEmbedder``
        Embedder for questions.
    action_embedding_dim : ``int``
        Dimension to use for action embeddings.
    encoder : ``Seq2SeqEncoder``
        The encoder to use for the input question.
    entity_encoder : ``Seq2VecEncoder``
        The encoder to used for averaging the words of an entity.
    max_decoding_steps : ``int``
        When we're decoding with a beam search, what's the maximum number of steps we should take?
        This only applies at evaluation time, not during training.
    use_neighbor_similarity_for_linking : ``bool``, optional (default=False)
        If ``True``, we will compute a max similarity between a question token and the `neighbors`
        of an entity as a component of the linking scores.  This is meant to capture the same kind
        of information as the ``related_column`` feature.
    dropout : ``float``, optional (default=0)
        If greater than 0, we will apply dropout with this probability after all encoders (pytorch
        LSTMs do not apply dropout to their last layer).
    num_linking_features : ``int``, optional (default=10)
        We need to construct a parameter vector for the linking features, so we need to know how
        many there are.  The default of 8 here matches the default in the ``KnowledgeGraphField``,
        which is to use all eight defined features. If this is 0, another term will be added to the
        linking score. This term contains the maximum similarity value from the entity's neighbors
        and the question.
    rule_namespace : ``str``, optional (default=rule_labels)
        The vocabulary namespace to use for production rules.  The default corresponds to the
        default used in the dataset reader, so you likely don't need to modify this.
    tables_directory : ``str``, optional (default=/wikitables/)
        The directory to find tables when evaluating logical forms.  We rely on a call to SEMPRE to
        evaluate logical forms, and SEMPRE needs to read the table from disk itself.  This tells
        SEMPRE where to find the tables.
    """
    # pylint: disable=abstract-method
    def __init__(self,
                 vocab            ,
                 question_embedder                   ,
                 action_embedding_dim     ,
                 encoder                ,
                 entity_encoder                ,
                 max_decoding_steps     ,
                 use_neighbor_similarity_for_linking       = False,
                 dropout        = 0.0,
                 num_linking_features      = 10,
                 rule_namespace      = u'rule_labels',
                 tables_directory      = u'/wikitables/')        :
        super(WikiTablesSemanticParser, self).__init__(vocab)
        self._question_embedder = question_embedder
        self._encoder = encoder
        self._entity_encoder = TimeDistributed(entity_encoder)
        self._max_decoding_steps = max_decoding_steps
        self._use_neighbor_similarity_for_linking = use_neighbor_similarity_for_linking
        if dropout > 0:
            self._dropout = torch.nn.Dropout(p=dropout)
        else:
            self._dropout = lambda x: x
        self._rule_namespace = rule_namespace
        self._denotation_accuracy = WikiTablesAccuracy(tables_directory)
        self._action_sequence_accuracy = Average()
        self._has_logical_form = Average()

        self._action_padding_index = -1  # the padding value used by IndexField
        num_actions = vocab.get_vocab_size(self._rule_namespace)
        self._action_embedder = Embedding(num_embeddings=num_actions, embedding_dim=action_embedding_dim)
        self._output_action_embedder = Embedding(num_embeddings=num_actions, embedding_dim=action_embedding_dim)
        self._action_biases = Embedding(num_embeddings=num_actions, embedding_dim=1)

        # This is what we pass as input in the first step of decoding, when we don't have a
        # previous action, or a previous question attention.
        self._first_action_embedding = torch.nn.Parameter(torch.FloatTensor(action_embedding_dim))
        self._first_attended_question = torch.nn.Parameter(torch.FloatTensor(encoder.get_output_dim()))
        torch.nn.init.normal_(self._first_action_embedding)
        torch.nn.init.normal_(self._first_attended_question)

        check_dimensions_match(entity_encoder.get_output_dim(), question_embedder.get_output_dim(),
                               u"entity word average embedding dim", u"question embedding dim")

        self._num_entity_types = 4  # TODO(mattg): get this in a more principled way somehow?
        self._num_start_types = 5  # TODO(mattg): get this in a more principled way somehow?
        self._embedding_dim = question_embedder.get_output_dim()
        self._type_params = torch.nn.Linear(self._num_entity_types, self._embedding_dim)
        self._neighbor_params = torch.nn.Linear(self._embedding_dim, self._embedding_dim)

        if num_linking_features > 0:
            self._linking_params = torch.nn.Linear(num_linking_features, 1)
        else:
            self._linking_params = None

        if self._use_neighbor_similarity_for_linking:
            self._question_entity_params = torch.nn.Linear(1, 1)
            self._question_neighbor_params = torch.nn.Linear(1, 1)
        else:
            self._question_entity_params = None
            self._question_neighbor_params = None

    def _get_initial_state_and_scores(self,
                                      question                             ,
                                      table                             ,
                                      world                       ,
                                      actions                                 ,
                                      example_lisp_string            = None,
                                      add_world_to_initial_state       = False,
                                      checklist_states                       = None)        :
        u"""
        Does initial preparation and creates an intiial state for both the semantic parsers. Note
        that the checklist state is optional, and the ``WikiTablesMmlParser`` is not expected to
        pass it.
        """
        table_text = table[u'text']
        # (batch_size, question_length, embedding_dim)
        embedded_question = self._question_embedder(question)
        question_mask = util.get_text_field_mask(question).float()
        # (batch_size, num_entities, num_entity_tokens, embedding_dim)
        embedded_table = self._question_embedder(table_text, num_wrapping_dims=1)
        table_mask = util.get_text_field_mask(table_text, num_wrapping_dims=1).float()

        batch_size, num_entities, num_entity_tokens, _ = embedded_table.size()
        num_question_tokens = embedded_question.size(1)

        # (batch_size, num_entities, embedding_dim)
        encoded_table = self._entity_encoder(embedded_table, table_mask)
        # (batch_size, num_entities, num_neighbors)
        neighbor_indices = self._get_neighbor_indices(world, num_entities, encoded_table)

        # Neighbor_indices is padded with -1 since 0 is a potential neighbor index.
        # Thus, the absolute value needs to be taken in the index_select, and 1 needs to
        # be added for the mask since that method expects 0 for padding.
        # (batch_size, num_entities, num_neighbors, embedding_dim)
        embedded_neighbors = util.batched_index_select(encoded_table, torch.abs(neighbor_indices))

        neighbor_mask = util.get_text_field_mask({u'ignored': neighbor_indices + 1},
                                                 num_wrapping_dims=1).float()

        # Encoder initialized to easily obtain a masked average.
        neighbor_encoder = TimeDistributed(BagOfEmbeddingsEncoder(self._embedding_dim, averaged=True))
        # (batch_size, num_entities, embedding_dim)
        embedded_neighbors = neighbor_encoder(embedded_neighbors, neighbor_mask)

        # entity_types: one-hot tensor with shape (batch_size, num_entities, num_types)
        # entity_type_dict: Dict[int, int], mapping flattened_entity_index -> type_index
        # These encode the same information, but for efficiency reasons later it's nice
        # to have one version as a tensor and one that's accessible on the cpu.
        entity_types, entity_type_dict = self._get_type_vector(world, num_entities, encoded_table)

        entity_type_embeddings = self._type_params(entity_types.float())
        projected_neighbor_embeddings = self._neighbor_params(embedded_neighbors.float())
        # (batch_size, num_entities, embedding_dim)
        entity_embeddings = torch.tanh(entity_type_embeddings + projected_neighbor_embeddings)


        # Compute entity and question word similarity.  We tried using cosine distance here, but
        # because this similarity is the main mechanism that the model can use to push apart logit
        # scores for certain actions (like "n -> 1" and "n -> -1"), this needs to have a larger
        # output range than [-1, 1].
        question_entity_similarity = torch.bmm(embedded_table.view(batch_size,
                                                                   num_entities * num_entity_tokens,
                                                                   self._embedding_dim),
                                               torch.transpose(embedded_question, 1, 2))

        question_entity_similarity = question_entity_similarity.view(batch_size,
                                                                     num_entities,
                                                                     num_entity_tokens,
                                                                     num_question_tokens)

        # (batch_size, num_entities, num_question_tokens)
        question_entity_similarity_max_score, _ = torch.max(question_entity_similarity, 2)

        # (batch_size, num_entities, num_question_tokens, num_features)
        linking_features = table[u'linking']

        linking_scores = question_entity_similarity_max_score

        if self._use_neighbor_similarity_for_linking:
            # The linking score is computed as a linear projection of two terms. The first is the
            # maximum similarity score over the entity's words and the question token. The second
            # is the maximum similarity over the words in the entity's neighbors and the question
            # token.
            #
            # The second term, projected_question_neighbor_similarity, is useful when a column
            # needs to be selected. For example, the question token might have no similarity with
            # the column name, but is similar with the cells in the column.
            #
            # Note that projected_question_neighbor_similarity is intended to capture the same
            # information as the related_column feature.
            #
            # Also note that this block needs to be _before_ the `linking_params` block, because
            # we're overwriting `linking_scores`, not adding to it.

            # (batch_size, num_entities, num_neighbors, num_question_tokens)
            question_neighbor_similarity = util.batched_index_select(question_entity_similarity_max_score,
                                                                     torch.abs(neighbor_indices))
            # (batch_size, num_entities, num_question_tokens)
            question_neighbor_similarity_max_score, _ = torch.max(question_neighbor_similarity, 2)
            projected_question_entity_similarity = self._question_entity_params(
                    question_entity_similarity_max_score.unsqueeze(-1)).squeeze(-1)
            projected_question_neighbor_similarity = self._question_neighbor_params(
                    question_neighbor_similarity_max_score.unsqueeze(-1)).squeeze(-1)
            linking_scores = projected_question_entity_similarity + projected_question_neighbor_similarity

        feature_scores = None
        if self._linking_params is not None:
            feature_scores = self._linking_params(linking_features).squeeze(3)
            linking_scores = linking_scores + feature_scores

        # (batch_size, num_question_tokens, num_entities)
        linking_probabilities = self._get_linking_probabilities(world, linking_scores.transpose(1, 2),
                                                                question_mask, entity_type_dict)

        # (batch_size, num_question_tokens, embedding_dim)
        link_embedding = util.weighted_sum(entity_embeddings, linking_probabilities)
        encoder_input = torch.cat([link_embedding, embedded_question], 2)

        # (batch_size, question_length, encoder_output_dim)
        encoder_outputs = self._dropout(self._encoder(encoder_input, question_mask))

        # This will be our initial hidden state and memory cell for the decoder LSTM.
        final_encoder_output = util.get_final_encoder_states(encoder_outputs,
                                                             question_mask,
                                                             self._encoder.is_bidirectional())
        memory_cell = encoder_outputs.new_zeros(batch_size, self._encoder.get_output_dim())

        initial_score = embedded_question.data.new_zeros(batch_size)

        action_embeddings, output_action_embeddings, action_biases, action_indices = self._embed_actions(actions)

        _, num_entities, num_question_tokens = linking_scores.size()
        flattened_linking_scores, actions_to_entities = self._map_entity_productions(linking_scores,
                                                                                     world,
                                                                                     actions)
        # To make grouping states together in the decoder easier, we convert the batch dimension in
        # all of our tensors into an outer list.  For instance, the encoder outputs have shape
        # `(batch_size, question_length, encoder_output_dim)`.  We need to convert this into a list
        # of `batch_size` tensors, each of shape `(question_length, encoder_output_dim)`.  Then we
        # won't have to do any index selects, or anything, we'll just do some `torch.cat()`s.
        initial_score_list = [initial_score[i] for i in range(batch_size)]
        encoder_output_list = [encoder_outputs[i] for i in range(batch_size)]
        question_mask_list = [question_mask[i] for i in range(batch_size)]
        initial_rnn_state = []
        for i in range(batch_size):
            initial_rnn_state.append(RnnState(final_encoder_output[i],
                                              memory_cell[i],
                                              self._first_action_embedding,
                                              self._first_attended_question,
                                              encoder_output_list,
                                              question_mask_list))
        initial_grammar_state = [self._create_grammar_state(world[i], actions[i])
                                 for i in range(batch_size)]
        initial_state_world = world if add_world_to_initial_state else None
        initial_state = WikiTablesDecoderState(batch_indices=range(batch_size),
                                               action_history=[[] for _ in range(batch_size)],
                                               score=initial_score_list,
                                               rnn_state=initial_rnn_state,
                                               grammar_state=initial_grammar_state,
                                               action_embeddings=action_embeddings,
                                               output_action_embeddings=output_action_embeddings,
                                               action_biases=action_biases,
                                               action_indices=action_indices,
                                               possible_actions=actions,
                                               flattened_linking_scores=flattened_linking_scores,
                                               actions_to_entities=actions_to_entities,
                                               entity_types=entity_type_dict,
                                               world=initial_state_world,
                                               example_lisp_string=example_lisp_string,
                                               checklist_state=checklist_states,
                                               debug_info=None)
        return {u"initial_state": initial_state,
                u"linking_scores": linking_scores,
                u"feature_scores": feature_scores,
                u"similarity_scores": question_entity_similarity_max_score}

    @staticmethod
    def _get_neighbor_indices(worlds                       ,
                              num_entities     ,
                              tensor              )                    :
        u"""
        This method returns the indices of each entity's neighbors. A tensor
        is accepted as a parameter for copying purposes.

        Parameters
        ----------
        worlds : ``List[WikiTablesWorld]``
        num_entities : ``int``
        tensor : ``torch.Tensor``
            Used for copying the constructed list onto the right device.

        Returns
        -------
        A ``torch.LongTensor`` with shape ``(batch_size, num_entities, num_neighbors)``. It is padded
        with -1 instead of 0, since 0 is a valid neighbor index.
        """

        num_neighbors = 0
        for world in worlds:
            for entity in world.table_graph.entities:
                if len(world.table_graph.neighbors[entity]) > num_neighbors:
                    num_neighbors = len(world.table_graph.neighbors[entity])

        batch_neighbors = []
        for world in worlds:
            # Each batch instance has its own world, which has a corresponding table.
            entities = world.table_graph.entities
            entity2index = dict((entity, i) for i, entity in enumerate(entities))
            entity2neighbors = world.table_graph.neighbors
            neighbor_indexes = []
            for entity in entities:
                entity_neighbors = [entity2index[n] for n in entity2neighbors[entity]]
                # Pad with -1 instead of 0, since 0 represents a neighbor index.
                padded = pad_sequence_to_length(entity_neighbors, num_neighbors, lambda: -1)
                neighbor_indexes.append(padded)
            neighbor_indexes = pad_sequence_to_length(neighbor_indexes,
                                                      num_entities,
                                                      lambda: [-1] * num_neighbors)
            batch_neighbors.append(neighbor_indexes)
        return tensor.new_tensor(batch_neighbors, dtype=torch.long)

    @staticmethod
    def _get_type_vector(worlds                       ,
                         num_entities     ,
                         tensor              )                                           :
        u"""
        Produces the one hot encoding for each entity's type. In addition,
        a map from a flattened entity index to type is returned to combine
        entity type operations into one method.

        Parameters
        ----------
        worlds : ``List[WikiTablesWorld]``
        num_entities : ``int``
        tensor : ``torch.Tensor``
            Used for copying the constructed list onto the right device.

        Returns
        -------
        A ``torch.LongTensor`` with shape ``(batch_size, num_entities, num_types)``.
        entity_types : ``Dict[int, int]``
            This is a mapping from ((batch_index * num_entities) + entity_index) to entity type id.
        """
        entity_types = {}
        batch_types = []
        for batch_index, world in enumerate(worlds):
            types = []
            for entity_index, entity in enumerate(world.table_graph.entities):
                one_hot_vectors = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]]
                # We need numbers to be first, then cells, then parts, then row, because our
                # entities are going to be sorted.  We do a split by type and then a merge later,
                # and it relies on this sorting.
                if entity.startswith(u'fb:cell'):
                    entity_type = 1
                elif entity.startswith(u'fb:part'):
                    entity_type = 2
                elif entity.startswith(u'fb:row'):
                    entity_type = 3
                else:
                    entity_type = 0
                types.append(one_hot_vectors[entity_type])

                # For easier lookups later, we're actually using a _flattened_ version
                # of (batch_index, entity_index) for the key, because this is how the
                # linking scores are stored.
                flattened_entity_index = batch_index * num_entities + entity_index
                entity_types[flattened_entity_index] = entity_type
            padded = pad_sequence_to_length(types, num_entities, lambda: [0, 0, 0, 0])
            batch_types.append(padded)
        return tensor.new_tensor(batch_types), entity_types

    def _get_linking_probabilities(self,
                                   worlds                       ,
                                   linking_scores                   ,
                                   question_mask                  ,
                                   entity_type_dict                )                     :
        u"""
        Produces the probability of an entity given a question word and type. The logic below
        separates the entities by type since the softmax normalization term sums over entities
        of a single type.

        Parameters
        ----------
        worlds : ``List[WikiTablesWorld]``
        linking_scores : ``torch.FloatTensor``
            Has shape (batch_size, num_question_tokens, num_entities).
        question_mask: ``torch.LongTensor``
            Has shape (batch_size, num_question_tokens).
        entity_type_dict : ``Dict[int, int]``
            This is a mapping from ((batch_index * num_entities) + entity_index) to entity type id.

        Returns
        -------
        batch_probabilities : ``torch.FloatTensor``
            Has shape ``(batch_size, num_question_tokens, num_entities)``.
            Contains all the probabilities for an entity given a question word.
        """
        _, num_question_tokens, num_entities = linking_scores.size()
        batch_probabilities = []

        for batch_index, world in enumerate(worlds):
            all_probabilities = []
            num_entities_in_instance = 0

            # NOTE: The way that we're doing this here relies on the fact that entities are
            # implicitly sorted by their types when we sort them by name, and that numbers come
            # before "fb:cell", and "fb:cell" comes before "fb:row".  This is not a great
            # assumption, and could easily break later, but it should work for now.
            for type_index in range(self._num_entity_types):
                # This index of 0 is for the null entity for each type, representing the case where a
                # word doesn't link to any entity.
                entity_indices = [0]
                entities = world.table_graph.entities
                for entity_index, _ in enumerate(entities):
                    if entity_type_dict[batch_index * num_entities + entity_index] == type_index:
                        entity_indices.append(entity_index)

                if len(entity_indices) == 1:
                    # No entities of this type; move along...
                    continue

                # We're subtracting one here because of the null entity we added above.
                num_entities_in_instance += len(entity_indices) - 1

                # We separate the scores by type, since normalization is done per type.  There's an
                # extra "null" entity per type, also, so we have `num_entities_per_type + 1`.  We're
                # selecting from a (num_question_tokens, num_entities) linking tensor on _dimension 1_,
                # so we get back something of shape (num_question_tokens,) for each index we're
                # selecting.  All of the selected indices together then make a tensor of shape
                # (num_question_tokens, num_entities_per_type + 1).
                indices = linking_scores.new_tensor(entity_indices, dtype=torch.long)
                entity_scores = linking_scores[batch_index].index_select(1, indices)

                # We used index 0 for the null entity, so this will actually have some values in it.
                # But we want the null entity's score to be 0, so we set that here.
                entity_scores[:, 0] = 0

                # No need for a mask here, as this is done per batch instance, with no padding.
                type_probabilities = torch.nn.functional.softmax(entity_scores, dim=1)
                all_probabilities.append(type_probabilities[:, 1:])

            # We need to add padding here if we don't have the right number of entities.
            if num_entities_in_instance != num_entities:
                zeros = linking_scores.new_zeros(num_question_tokens,
                                                 num_entities - num_entities_in_instance)
                all_probabilities.append(zeros)

            # (num_question_tokens, num_entities)
            probabilities = torch.cat(all_probabilities, dim=1)
            batch_probabilities.append(probabilities)
        batch_probabilities = torch.stack(batch_probabilities, dim=0)
        return batch_probabilities * question_mask.unsqueeze(-1).float()

    @staticmethod
    def _action_history_match(predicted           , targets                  )       :
        # TODO(mattg): this could probably be moved into a FullSequenceMatch metric, or something.
        # Check if target is big enough to cover prediction (including start/end symbols)
        if len(predicted) > targets.size(1):
            return 0
        predicted_tensor = targets.new_tensor(predicted)
        targets_trimmed = targets[:, :len(predicted)]
        # Return 1 if the predicted sequence is anywhere in the list of targets.
        return torch.max(torch.min(targets_trimmed.eq(predicted_tensor), dim=1)[0]).item()

    #overrides
    def get_metrics(self, reset       = False)                    :
        u"""
        We track three metrics here:

            1. dpd_acc, which is the percentage of the time that our best output action sequence is
            in the set of action sequences provided by DPD.  This is an easy-to-compute lower bound
            on denotation accuracy for the set of examples where we actually have DPD output.  We
            only score dpd_acc on that subset.

            2. denotation_acc, which is the percentage of examples where we get the correct
            denotation.  This is the typical "accuracy" metric, and it is what you should usually
            report in an experimental result.  You need to be careful, though, that you're
            computing this on the full data, and not just the subset that has DPD output (make sure
            you pass "keep_if_no_dpd=True" to the dataset reader, which we do for validation data,
            but not training data).

            3. lf_percent, which is the percentage of time that decoding actually produces a
            finished logical form.  We might not produce a valid logical form if the decoder gets
            into a repetitive loop, or we're trying to produce a super long logical form and run
            out of time steps, or something.
        """
        return {
                u'dpd_acc': self._action_sequence_accuracy.get_metric(reset),
                u'denotation_acc': self._denotation_accuracy.get_metric(reset),
                u'lf_percent': self._has_logical_form.get_metric(reset),
                }

    @staticmethod
    def _create_grammar_state(world                 ,
                              possible_actions                           )                :
        valid_actions = world.get_valid_actions()
        action_mapping = {}
        for i, action in enumerate(possible_actions):
            action_string = action[0]
            action_mapping[action_string] = i
        translated_valid_actions = {}
        for key, action_strings in list(valid_actions.items()):
            translated_valid_actions[key] = [action_mapping[action_string]
                                             for action_string in action_strings]
        return GrammarState([START_SYMBOL],
                            {},
                            translated_valid_actions,
                            action_mapping,
                            type_declaration.is_nonterminal)

    def _embed_actions(self, actions                                 ):                       
                                                                                             
                                                                                             
                                                                                                           
        u"""
        Given all of the possible actions for all batch instances, produce an embedding for them.
        There will be significant overlap in this list, as the production rules from the grammar
        are shared across all batch instances.  Our returned tensor has an embedding for each
        `unique` action, so we also need to return a mapping from the original ``(batch_index,
        action_index)`` to our new ``global_action_index``, so that we can get the right action
        embedding during decoding.

        Returns
        -------
        action_embeddings : ``torch.Tensor``
            Has shape ``(num_unique_actions, action_embedding_dim)``.
        output_action_embeddings : ``torch.Tensor``
            Has shape ``(num_unique_actions, action_embedding_dim)``.
        action_biases : ``torch.Tensor``
            Has shape ``(num_unique_actions, 1)``.
        action_map : ``Dict[Tuple[int, int], int]``
            Maps ``(batch_index, action_index)`` in the input action list to ``action_index`` in
            the ``action_embeddings`` tensor.  All non-embeddable actions get mapped to `-1` here.
        """
        # TODO(mattg): This whole action pipeline might be a whole lot more complicated than it
        # needs to be.  We used to embed actions differently (using some crazy ideas about
        # embedding the LHS and RHS separately); we could probably get away with simplifying things
        # further now that we're just doing a simple embedding for global actions.  But I'm leaving
        # it like this for now to have a minimal change to go from the LHS/RHS embedding to a
        # single action embedding.
        embedded_actions = self._action_embedder.weight
        output_embedded_actions = self._output_action_embedder.weight
        action_biases = self._action_biases.weight

        # Now we just need to make a map from `(batch_index, action_index)` to
        # `global_action_index`.  global_action_ids has the list of all unique actions; here we're
        # going over all of the actions for each batch instance so we can map them to the global
        # action ids.
        action_vocab = self.vocab.get_token_to_index_vocabulary(self._rule_namespace)
        action_map                             = {}
        for batch_index, instance_actions in enumerate(actions):
            for action_index, action in enumerate(instance_actions):
                if not action[0]:
                    # This rule is padding.
                    continue
                global_action_id = action_vocab.get(action[0], -1)
                action_map[(batch_index, action_index)] = global_action_id
        return embedded_actions, output_embedded_actions, action_biases, action_map

    @staticmethod
    def _map_entity_productions(linking_scores                   ,
                                worlds                       ,
                                actions                                 ):                       
                                                                                                              
        u"""
        Constructs a map from ``(batch_index, action_index)`` to ``(batch_index * entity_index)``.
        That is, some actions correspond to terminal productions of entities from our table.  We
        need to find those actions and map them to their corresponding entity indices, where the
        entity index is its position in the list of entities returned by the ``world``.  This list
        is what defines the second dimension of the ``linking_scores`` tensor, so we can use this
        index to look up linking scores for each action in that tensor.

        For easier processing later, the mapping that we return is `flattened` - we really want to
        map ``(batch_index, action_index)`` to ``(batch_index, entity_index)``, but we are going to
        have to use the result of this mapping to do ``index_selects`` on the ``linking_scores``
        tensor.  You can't do ``index_select`` with tuples, so we flatten ``linking_scores`` to
        have shape ``(batch_size * num_entities, num_question_tokens)``, and return shifted indices
        into this flattened tensor.

        Parameters
        ----------
        linking_scores : ``torch.Tensor``
            A tensor representing linking scores between each table entity and each question token.
            Has shape ``(batch_size, num_entities, num_question_tokens)``.
        worlds : ``List[WikiTablesWorld]``
            The ``World`` for each batch instance.  The ``World`` contains a reference to the
            ``TableKnowledgeGraph`` that defines the set of entities in the linking.
        actions : ``List[List[ProductionRuleArray]]``
            The list of possible actions for each batch instance.  Our action indices are defined
            in terms of this list, so we'll find entity productions in this list and map them to
            entity indices from the entity list we get from the ``World``.

        Returns
        -------
        flattened_linking_scores : ``torch.Tensor``
            A flattened version of ``linking_scores``, with shape ``(batch_size * num_entities,
            num_question_tokens)``.
        actions_to_entities : ``Dict[Tuple[int, int], int]``
            A mapping from ``(batch_index, action_index)`` to ``(batch_size * num_entities)``,
            representing which action indices correspond to which entity indices in the returned
            ``flattened_linking_scores`` tensor.
        """
        batch_size, num_entities, num_question_tokens = linking_scores.size()
        entity_map                             = {}
        for batch_index, world in enumerate(worlds):
            for entity_index, entity in enumerate(world.table_graph.entities):
                entity_map[(batch_index, entity)] = batch_index * num_entities + entity_index
        actions_to_entities                             = {}
        for batch_index, action_list in enumerate(actions):
            for action_index, action in enumerate(action_list):
                if not action[0]:
                    # This action is padding.
                    continue
                _, production = action[0].split(u' -> ')
                entity_index = entity_map.get((batch_index, production), None)
                if entity_index is not None:
                    actions_to_entities[(batch_index, action_index)] = entity_index
        flattened_linking_scores = linking_scores.view(batch_size * num_entities, num_question_tokens)
        return flattened_linking_scores, actions_to_entities

    #overrides
    def decode(self, output_dict                         )                           :
        u"""
        This method overrides ``Model.decode``, which gets called after ``Model.forward``, at test
        time, to finalize predictions.  This is (confusingly) a separate notion from the "decoder"
        in "encoder/decoder", where that decoder logic lives in ``WikiTablesDecoderStep``.

        This method trims the output predictions to the first end symbol, replaces indices with
        corresponding tokens, and adds a field called ``predicted_tokens`` to the ``output_dict``.
        """
        action_mapping = output_dict[u'action_mapping']
        best_actions = output_dict[u"best_action_sequence"]
        debug_infos = output_dict[u'debug_info']
        batch_action_info = []
        for batch_index, (predicted_actions, debug_info) in enumerate(izip(best_actions, debug_infos)):
            instance_action_info = []
            for predicted_action, action_debug_info in izip(predicted_actions, debug_info):
                action_info = {}
                action_info[u'predicted_action'] = predicted_action
                considered_actions = action_debug_info[u'considered_actions']
                probabilities = action_debug_info[u'probabilities']
                actions = []
                for action, probability in izip(considered_actions, probabilities):
                    if action != -1:
                        actions.append((action_mapping[(batch_index, action)], probability))
                actions.sort()
                considered_actions, probabilities = izip(*actions)
                action_info[u'considered_actions'] = considered_actions
                action_info[u'action_probabilities'] = probabilities
                action_info[u'question_attention'] = action_debug_info.get(u'question_attention', [])
                instance_action_info.append(action_info)
            batch_action_info.append(instance_action_info)
        output_dict[u"predicted_actions"] = batch_action_info
        return output_dict



# pylint: disable=no-self-use,invalid-name


from __future__ import absolute_import
import numpy
from numpy.testing import assert_almost_equal
import torch
from torch.nn.parameter import Parameter

from allennlp.common import Params
from allennlp.modules.similarity_functions import BilinearSimilarity
from allennlp.common.testing import AllenNlpTestCase

class TestBilinearSimilarityFunction(AllenNlpTestCase):
    def test_weights_are_correct_sizes(self):
        # pylint: disable=protected-access
        bilinear = BilinearSimilarity(tensor_1_dim=5, tensor_2_dim=2)
        assert list(bilinear._weight_matrix.size()) == [5, 2]
        assert list(bilinear._bias.size()) == [1]

    def test_forward_does_a_bilinear_product(self):
        # pylint: disable=protected-access
        bilinear = BilinearSimilarity(2, 2)
        bilinear._weight_matrix = Parameter(torch.FloatTensor([[-.3, .5], [2.0, -1.0]]))
        bilinear._bias = Parameter(torch.FloatTensor([.1]))
        a_vectors = torch.FloatTensor([[1, 1], [-1, -1]])
        b_vectors = torch.FloatTensor([[1, 0], [0, 1]])
        result = bilinear(a_vectors, b_vectors).data.numpy()
        assert result.shape == (2,)
        assert_almost_equal(result, [1.8, .6])

    def test_forward_works_with_higher_order_tensors(self):
        # pylint: disable=protected-access
        bilinear = BilinearSimilarity(4, 7)
        weights = numpy.random.rand(4, 7)
        bilinear._weight_matrix = Parameter(torch.from_numpy(weights).float())
        bilinear._bias = Parameter(torch.from_numpy(numpy.asarray([0])).float())
        a_vectors = numpy.random.rand(5, 4, 3, 6, 4)
        b_vectors = numpy.random.rand(5, 4, 3, 6, 7)
        a_variables = torch.from_numpy(a_vectors).float()
        b_variables = torch.from_numpy(b_vectors).float()
        result = bilinear(a_variables, b_variables).data.numpy()
        assert result.shape == (5, 4, 3, 6)
        expected_result = numpy.dot(numpy.dot(numpy.transpose(a_vectors[3, 2, 1, 3]), weights),
                                    b_vectors[3, 2, 1, 3])
        assert_almost_equal(result[3, 2, 1, 3], expected_result, decimal=5)

    def test_can_construct_from_params(self):
        params = Params({
                u'tensor_1_dim': 3,
                u'tensor_2_dim': 4
                })
        bilinear = BilinearSimilarity.from_params(params)
        assert list(bilinear._weight_matrix.size()) == [3, 4]  # pylint: disable=protected-access



# pylint: disable=no-self-use,invalid-name


from __future__ import absolute_import
import numpy
from numpy.testing import assert_almost_equal
import torch
from torch.nn.parameter import Parameter

from allennlp.common import Params
from allennlp.modules.similarity_functions import LinearSimilarity
from allennlp.common.testing import AllenNlpTestCase

class TestLinearSimilarityFunction(AllenNlpTestCase):
    # pylint: disable=protected-access
    def test_weights_are_correct_sizes(self):
        linear = LinearSimilarity(tensor_1_dim=3, tensor_2_dim=6, combination=u'x,y')
        assert list(linear._weight_vector.size()) == [9]
        assert list(linear._bias.size()) == [1]

    def test_forward_does_a_weighted_product(self):
        linear = LinearSimilarity(3, 1, combination=u'x,y')
        linear._weight_vector = Parameter(torch.FloatTensor([-.3, .5, 2.0, -1.0]))
        linear._bias = Parameter(torch.FloatTensor([.1]))
        a_vectors = torch.FloatTensor([[[1, 1, 1], [-1, -1, 0]]])
        b_vectors = torch.FloatTensor([[[0], [1]]])
        result = linear(a_vectors, b_vectors).data.numpy()
        assert result.shape == (1, 2,)
        assert_almost_equal(result, [[2.3, -1.1]])

    def test_forward_works_with_higher_order_tensors(self):
        linear = LinearSimilarity(7, 7, combination=u'x,y')
        weights = numpy.random.rand(14)
        linear._weight_vector = Parameter(torch.from_numpy(weights).float())
        linear._bias = Parameter(torch.FloatTensor([0.]))
        a_vectors = numpy.random.rand(5, 4, 3, 6, 7)
        b_vectors = numpy.random.rand(5, 4, 3, 6, 7)
        result = linear(torch.from_numpy(a_vectors).float(),
                        torch.from_numpy(b_vectors).float())
        result = result.data.numpy()
        assert result.shape == (5, 4, 3, 6)
        combined_vectors = numpy.concatenate([a_vectors[3, 2, 1, 3, :], b_vectors[3, 2, 1, 3, :]])
        expected_result = numpy.dot(combined_vectors, weights)
        assert_almost_equal(result[3, 2, 1, 3], expected_result, decimal=6)

    def test_forward_works_with_multiply_combinations(self):
        linear = LinearSimilarity(2, 2, combination=u'x*y')
        linear._weight_vector = Parameter(torch.FloatTensor([-.3, .5]))
        linear._bias = Parameter(torch.FloatTensor([0]))
        a_vectors = torch.FloatTensor([[1, 1], [-1, -1]])
        b_vectors = torch.FloatTensor([[1, 0], [0, 1]])
        result = linear(a_vectors, b_vectors).data.numpy()
        assert result.shape == (2,)
        assert_almost_equal(result, [-.3, -.5])

    def test_forward_works_with_divide_combinations(self):
        linear = LinearSimilarity(2, 2, combination=u'x/y')
        linear._weight_vector = Parameter(torch.FloatTensor([-.3, .5]))
        linear._bias = Parameter(torch.FloatTensor([0]))
        a_vectors = torch.FloatTensor([[1, 1], [-1, -1]])
        b_vectors = torch.FloatTensor([[1, 2], [2, 1]])
        result = linear(a_vectors, b_vectors).data.numpy()
        assert result.shape == (2,)
        assert_almost_equal(result, [-.05, -.35])

    def test_forward_works_with_add_combinations(self):
        linear = LinearSimilarity(2, 2, combination=u'x+y')
        linear._weight_vector = Parameter(torch.FloatTensor([-.3, .5]))
        linear._bias = Parameter(torch.FloatTensor([0]))
        a_vectors = torch.FloatTensor([[1, 1], [-1, -1]])
        b_vectors = torch.FloatTensor([[1, 0], [0, 1]])
        result = linear(a_vectors, b_vectors).data.numpy()
        assert result.shape == (2,)
        assert_almost_equal(result, [-.1, .3])

    def test_forward_works_with_subtract_combinations(self):
        linear = LinearSimilarity(2, 2, combination=u'x-y')
        linear._weight_vector = Parameter(torch.FloatTensor([-.3, .5]))
        linear._bias = Parameter(torch.FloatTensor([0]))
        a_vectors = torch.FloatTensor([[1, 1], [-1, -1]])
        b_vectors = torch.FloatTensor([[1, 0], [0, 1]])
        result = linear(a_vectors, b_vectors).data.numpy()
        assert result.shape == (2,)
        assert_almost_equal(result, [.5, -.7])

    def test_can_construct_from_params(self):
        params = Params({
                u'tensor_1_dim': 4,
                u'tensor_2_dim': 4,
                u'combination': u'x,y,x*y,y-x'
                })
        linear = LinearSimilarity.from_params(params)
        assert list(linear._weight_vector.size()) == [16]
